@@ -8,7 +8,11 @@
      (extended-bindings)
      (block)
      (not safe)))
-  (else))
+  (else
+   (import (chicken bitwise)
+	   (chicken fixnum)
+	   (chicken flonum)
+	   (chicken process-context))))
 
 ;;; All the following redefinitions are *ignored* by the Gambit compiler
 ;;; because of the declarations above.
@@ -55,7 +59,7 @@
     (defalias list->f64vector list->vector)
     (defalias f64vector-length vector-length)) )
  (chicken
-  (use srfi-4))
+  (import srfi-4))
  (else) )
 
 ;;; end of *ignored* definitions
@@ -1633,7 +1637,6 @@
 	  (define (extend-lut multiplier-lut bit-reverse-size bit-reverse-multiplier start end)
 
 	    (define (bit-reverse x n)
-	      (declare (not interrupts-enabled))
 	      (do ((i 0 (fx+ i 1))
 		   (x x (fxarithmetic-shift-right x 1))
 		   (result 0 (fx+ (fx* result 2)
@@ -1868,8 +1871,6 @@
       ;; two pass with w=1 (so W[0]=1.0 and W[1] = 0.)  and then
       ;; call recursive-bit appropriately on the two half arrays.
 
-      (declare (not interrupts-enabled))
-
       (let ((SizeOfGroup
 	     (fxarithmetic-shift-right (f64vector-length a) 1)))
 	(let loop ((J0 0))
@@ -2034,7 +2035,6 @@
 	    (main-loop M N K SizeOfGroup))))
 
     (define (radix-2-pass a)
-      (declare (not interrupts-enabled))
       (let ((SizeOfGroup
 	     (fxarithmetic-shift-right (f64vector-length a) 1)))
 	(let loop ((J0 0))
