@@ -1,14 +1,18 @@
 ;;;; private-repository-test.scm
 
 
-(use files posix)
+(import (chicken pathname)
+	(chicken platform)
+        (chicken process-context)
+        (chicken file)
+        (chicken file posix))
 
 (define read-symbolic-link*
   (cond-expand
    ((and windows (not cygwin)) (lambda (filename _) filename))
    (else read-symbolic-link)))
 
-(define repo (normalize-pathname (read-symbolic-link* (repository-path) #t)))
+(define repo (normalize-pathname (read-symbolic-link* (car (repository-path)) #t)))
 (define dir (normalize-pathname (read-symbolic-link* (car (command-line-arguments)) #t)))
 
 (print (list dir repo))
