@@ -21,3 +21,15 @@
   (assert (= 100 (get-z f)))
   (set! (make-foo f) 1000)
   (assert (= 1000 (get-t f))))
+
+(define-record bar
+  a
+  (setter b))
+
+(let ((b (make-bar 1 2)))
+  (assert (bar? b))
+  (bar-a-set! b 3)
+  (assert (= 3 (bar-a b)))
+  (setter bar-b)
+  (set! (bar-b b) 4)
+  (assert (= (bar-b b) 4)))
