@@ -1,6 +1,9 @@
 ;;;; test-finalizers.scm
 
-(use extras)
+;; NOTE: This may fail, reopen #1426 if it does
+
+(import (chicken format))
+(import (chicken gc))
 
 (##sys#eval-debug-level 0)		; disable keeping trace-buffer with frameinfo
 
@@ -69,6 +72,10 @@ a fix that unfortunately disables finalizers in the interpreter
 ;; Finalizers on constants are ignored in compiled mode (because
 ;; they're never GCed).  Reported by "Pluijzer".
 
+#| this doesn't always work in csi, for some unknown reason,
+   depending on unrelated factors (command-line options,
+   memory usage patterns, etc.)
+                                                
 (set! n 0)
 (define bar "constant string")
 (set-finalizer! bar bump)
@@ -78,3 +85,4 @@ a fix that unfortunately disables finalizers in the interpreter
 (cond-expand
   (compiling (assert (= 0 n)))
   (else (assert (= 1 n))))
+|#
