@@ -5,11 +5,10 @@
 
 
 (functor (test-queue (Q QUEUE)) *
-  (import scheme chicken Q)
-  (use srfi-1)
+  (import scheme (chicken base) Q)
   
   (define (list->queue lst)
-    (fold (lambda (x q) (enqueue q x)) empty-queue lst))
+    (foldl (lambda (q x) (enqueue q x)) empty-queue lst))
 
   (define (queue->list q)
     (if (empty? q)
